/* 
   audio_core_common.c - common stuff for, at least, OSS and ALSA cores

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>
#include <pthread.h>
#include <signal.h>

#include "config.h"
#ifdef HAVE_ALSA
#define ALSA_PCM_NEW_HW_PARAMS_API
#define ALSA_PCM_NEW_SW_PARAMS_API
#include <alsa/asoundlib.h>
#endif

#include "nvrec.h"

struct frag_t {
	unsigned char *buf;	// data buffer
	ntime_t time;		// timestamp
	int full;		// buffer full
};

struct audio_core_common_t {
	int fd;			// OSS file descriptor
#ifdef HAVE_ALSA
	snd_pcm_t *handle;	// ALSA handle
	snd_pcm_uframes_t period_size;
	unsigned int bits_per_frame;
#endif
	
	int rate;		// sampling rate requested
	int real_rate;		// sampling rate got
	int bits;		// sample bits
	int chans;		// no. channels

	int bps;		// bytes per second
	int realbps;		// bytes per second
	int fragsize;		// size of each fragment
	int nfrags;		// number of fragments in buffer
	
	struct frag_t *frag;	// audio buffer fragments
	int head;		// buffer head
	int tail;		// buffer tail
	int exit;		// stop audio thread
	int done;		// exited succesfully
};

void audio_core_freefrags(volatile struct audio_core_common_t * accm);
ntime_t audio_core_tailtime(struct aud_core_t * audt);
void audio_core_droptail(struct aud_core_t * audt);
int audio_core_read(struct aud_core_t * audt, ntime_t t, int drate, unsigned char * buf, int n);
unsigned long long audio_core_bps(struct aud_core_t * audt);
int audio_core_has_data(struct aud_core_t * audt);
int audio_core_fragsize(struct aud_core_t * audt);


